import numpy as np
import sys
import os
import re
import h5py
import torch
import time

def calcu_mp2(num_orb, lumo, noe, int2el, device):

  e1 = 0
  #ee = torch.zeros((lumo, lumo, num_orb-lumo, num_orb-lumo), device=device)
  ee = 2*int2el[:lumo,:lumo,lumo:,lumo:]-np.swapaxes(int2el[:lumo,:lumo,lumo:,lumo:],0,1)
   
  for r in range(lumo,num_orb):
    for s in range(lumo,num_orb):
      for a in range(lumo):
        for b in range(lumo):
          e1 = e1 + ee[a,b,r-lumo,s-lumo]*int2el[a,b,r,s]/(noe[a]+noe[b]-noe[r]-noe[s])

  amp = ee/(noe[:lumo,None,None,None]+noe[None,:lumo,None,None]-noe[None,None,lumo:num_orb,None]-noe[None,None,None,lumo:num_orb])
  
  return e1, amp


def main():  
  fwfn = sys.argv[1]
  ffci = sys.argv[2]
    
  # load orbital information
  with h5py.File(fwfn, 'r') as hf:
    typ_orb, num_orb, num_prt, num_nuc = hf['BASIC'][:]
    noon = hf['NOON'][:]
    noe = hf['NOE'][:]

  # load 2-ele terms
  int2el = np.load(ffci)

  # check lumo
  lumo = int(round(np.sum(noon)/2))
  if lumo == num_orb:
    print("Fully occupied, no need for MP3")
  if num_orb > len(int2el):
    print("FCIDUMP only has " + str(len(int2el)) + " orbitals, while WFN gives " + str(num_orb) + " orbitals, use FCIDUMP instead")
    num_orb = len(int2el)

  # calculate MP2 energy
  t1 = time.time()
  torch.set_num_threads(8)
  device = torch.device("cpu")
  int2el =torch.tensor(int2el, device=device).float()
  noe =torch.tensor(noe, device=device).float()

  emp2, amp = calcu_mp2(num_orb, lumo, noe, int2el, device)
  #print(time.time()-t1)
  #print(emp2)
          
  np.save('mp2_amp.npy', amp)
          

if __name__ == "__main__":
  main()
