import numpy as np
import sys
import os
import re
import h5py
import torch

def calcu_mp2(num_orb, lumo, noe, int2el, device):

  e1 = 0
  ee = torch.zeros((lumo, lumo, num_orb-lumo, num_orb-lumo), device=device)

  y = 2*int2el[:lumo,:lumo,lumo:,lumo:]-np.swapaxes(int2el[:lumo,:lumo,lumo:,lumo:],0,1)
  de = noe[:lumo,None,None,None] + noe[None,:lumo,None,None] - noe[None,None,lumo:,None] - noe[None,None,None,lumo:]
  e = np.einsum('ni,na,njb,ijab,ijab->n', wfn[:,:lumo], wfn[:,lumo:], A[:,:lumo,lumo:], tmp2, 1/de)
   
  for r in range(lumo,num_orb):
    for s in range(lumo,num_orb):
      for a in range(lumo):
        for b in range(lumo):
          e1 = e1 + ee[a,b,r-lumo,s-lumo]*int2el[a,b,r,s]/(noe[a]+noe[b]-noe[r]-noe[s])

  amp = ee/(noe[a,None,None,None]+noe[None,b,None,None]-noe[None,None,i,None]-noe[None,None,None,j])
  
  return e1.cpu().numpy(), amp.cpu().numpy()


def main():  
  fwfn = sys.argv[1]
  ffci = sys.argv[2]
    
  # load orbital information
  with h5py.File(fwfn, 'r') as hf:
    typ_orb, num_orb, num_prt, num_nuc = hf['BASIC'][:]
    noon = hf['NOON'][:]
    noe = hf['NOE'][:]

  # load 2-ele terms
  int2el = np.load(ffci)

  # check lumo
  lumo = int(round(np.sum(noon)/2))
  if lumo == num_orb:
    print("Fully occupied, no need for MP3")
  if num_orb > len(int2el):
    print("FCIDUMP only has " + str(len(int2el)) + " orbitals, while WFN gives " + str(num_orb) + " orbitals, use FCIDUMP instead")
    num_orb = len(int2el)

  # calculate MP2 energy
  t1 = time.time()
  gpu = torch.device("cuda:0")
  int2el =torch.tensor(int2el, device=gpu).float()
  noe =torch.tensor(noe, device=gpu).float()

  emp2, amp = calcu_mp2(num_orb, lumo, noe, int2el, gpu)
  print(time.time()-t1)
  print(emp2)
          
  np.save('mp2_amp.npy', amp)
          

if __name__ == "__main__":
  main()
